<?php

/**
 * "You can solve every problem with another level of indirection."
 * 
 * @package Broken Link Checker
 */
class blcActionDispatcher {
	
	/**
	 * Execute the handler associated with a specific action.
	 * 
	 * @param string $action The action to execute.
	 * @return bool|void
	 */
	function dispatch($action = ''){
		//Handler get?
		$className = blcActionDispatcher::action_to_classname($action);
		if ( !class_exists($className) ){
			return false;
		}
		
		//Bulk actions all come from one form, so they all use the same nonce field.
		if ( strpos($action, 'bulk-') === 0 ){
			check_admin_referer('bulk-action');
		} else {
			check_admin_referer($action);
		}
		
		$result = call_user_func(array($className, 'execute')); //May wp_die() if something goes wrong
		
		//Go back. URL params 'did_action' & 'result' are used to pass state and/or
		//action results to the report() method.
		$redirect = remove_query_arg(array('did_action', 'result'), wp_get_referer());
		$redirect = add_query_arg('did_action', $action, $redirect);
		if ( is_array($result) ){
			$redirect = add_query_arg('result', http_build_query($result, null, '&'), $redirect);
		}
		
		wp_redirect($redirect);
		die();
	}
	
	/**
	 * Generate status message(s) for a previously executed action.
	 * 
	 * @param string $action
	 * @return array List of messages to display. Each item will be an array comprising the message text as the first element and the corresponding CSS class as the second.
	 */
	function report($action = ''){
		$className = blcActionDispatcher::action_to_classname($action);
		if ( !class_exists($className) ){
			return array();
		}
		
		$state = array();
		if ( isset($_GET['result']) ){
			@parse_str($_GET['result'], $state);
		}
		
		return call_user_func(array($className, 'report'), $state);
	}
	
	/**
	 * Get the name of the handler class responsible for a particular action.
	 * 
	 * This function only generates the class name; it doesn't check if the 
	 * handler class actually exists.
	 * 
	 * @param string $action Action id, e.g. 'bulk-unlink'
	 * @return string Handler class name, e.g. 'blcDoBulkUnlink'
	 */
	function action_to_classname($action){
		//bulk-delete-sources => blcDoBulkDeleteSources
		$className = ucwords(str_replace(array('-', '_'), ' ', $action));
		return 'blcDo'. str_replace(' ', '', $className);
	}
	
	/**
	 * Retrieve the list of selected links.
	 * 
	 * @uses $_POST Reads link IDs from the 'selected_links' field of the link form in Tools -> Broken Links.
	 * @uses $_GET Alternatively, reads link IDs from the 'selected_links' query parameter.
	 * 
	 * @return array Array of numeric link IDs.
	 */
	function selected_link_ids(){
		$selected_links = array();
		
		if ( isset($_POST['selected_links']) && is_array($_POST['selected_links']) ){
			$selected_links = $_POST['selected_links'];
		} elseif ( isset($_GET['selected_links']) ){
			if ( is_array($_GET['selected_links']) ){
				$selected_links = $_GET['selected_links'];
			} else {
				$selected_links = explode(',', $_GET['selected_links']);
			}
		}
		
		//Convert all link IDs to integers (non-numeric entries are converted to zero)
		$selected_links = array_map('intval', $_POST['selected_links']);
		//Remove all zeroes
		$selected_links = array_filter($selected_links);
		
		return $selected_links;
	}
	
	/**
	 * Retrieve the list of selected containers.
	 * 
	 * @uses $_GET Reads the list of containers from the query string argument 'containers' 
	 * 
	 * @return array Array of [container_type, container_id] tuples identifying one or more containers
	 */
	function selected_containers(){
		$containers = array();
		if ( isset($_REQUEST['containers']) && !empty($_REQUEST['containers']) ){
			$containers = explode(',', $_REQUEST['containers']);
			$containers = array_map(
				create_function('$item', 'return explode("|", $item)'),
				$containers
			);
		}
		return $containers;
	}
}

/**
 * Base class for action handlers.
 * 
 * @package Broken Link Checker
 * @author janis Elsts
 */
class blcAbstractAction {
	
	/**
	 * Execute the action.
	 * 
	 * Since the return value(s) of this function will be serialized as query arguments,
	 * only primitive datatypes such as strings and numbers are safe to use. Everything 
	 * else may not get serialized properly.
	 * 
	 * @return array|void Either an associative array of values to be passed to the report() function, or nothing.   
	 */
	function execute(){
		//nil		
	}
	
	/**
	 * Generate status message(s) based on the results of the previously executed action.
	 * 
	 * The return value should be an array structured like this :
	 * array(
	 *    array('Success message', 'updated'),
	 *    array('Error message', 'error'),
	 *    ...
	 * )
	 * 
	 * @param $state array The associative array returned by execute().
	 * @return array List of messages to display. Each item will be an array comprising the message text as the first element and the corresponding CSS class as the second.
	 */
	function report($state){
		return array();
	}	
}



?>